==> pytactoe-0.8.0.tar.gz: pytactoe-0.8.0/pytactoe/variants.py <==
from pytactoe.board import Board, Piece
from pytactoe.tictactoe import TicTacToe


class WildTicTacToe(TicTacToe):
    ''' Tic Tac Toe where you can choose which piece to place. '''
    def __init__(self, m=3, n=3, k=3, infinite=False, first_player: Piece = Piece(1)) -> None:
        super().__init__(m, n, k, infinite, first_player)
        self.move_history_pieces = [] # Which pieces were chosen on each move

    def move(self, x, y, piece: Piece) -> bool:
        self.turn = piece
        if piece == Piece():
            return False
        was_played = super().move(x, y)
        self.move_history_pieces.append(piece)
        return was_played


class MisereTicTacToe(TicTacToe):
    '''Misere TicTacToe, where the player wins if the opponent 'wins' in normal rules. '''
    def __init__(self, m=3, n=3, k=3, infinite=False, first_player: Piece = Piece(1)) -> None:
        super().__init__(m, n, k, infinite, first_player)
    
    def checkwin(self) -> bool:
        if super().checkwin():
            self.winner = Piece.previous(self.turn)


class ImpartialTicTacToe(TicTacToe):
    ''' Tic Tac Toe with only one piece type. '''
    def __init__(self, m=3, n=3, k=3, infinite=False, first_player: Piece = Piece(1)) -> None:
        super().__init__(m, n, k, infinite, first_player)

    def move(self, x, y) -> bool:
        self.turn = Piece(1)
        super().move(x, y)


class UltimateTicTacToe(TicTacToe):
    ''' Ultimate TicTacToe. '''
    def __init__(self, m=3, n=3, k=3, where=(0, 0)) -> None:
        '''
        Initialize Ultimate TicTacToe.
        localboards attribute contain regular local boards; attribute .turn for local is unused.
        board attribute contain information about who won for each local board.
        where attribute determine on which local board the next move is pushed.
        where_history attribute contain all local boards coordinates.
        '''
        super().__init__(m, n, k)
        self.localboards = [[TicTacToe(m, n, k) for _ in range(m)] for _ in range(n)]
        self.where = where
        self.where_history = [where]

    def movevalid(self, x, y) -> bool:
        ''' Return True if move is valid. Note that we can move on already winned small boards. '''
        return self.localboards[self.where[0]][self.where[1]].movevalid(x, y)

    def move(self, x, y) -> bool:
        '''Make move, update attributes, choose small board for next move. Return True if move is successful. '''
        if self.movevalid(x, y):

            localboard = self.localboards[self.where[0]][self.where[1]]
            localboard[x, y] = self.turn
            localboard.move_history.append((x,y))
            localboard.move_count += 1

            self.move_history.append((x,y))
            self.move_count += 1
            
            self.where = (x, y)
            self.where_history.append(self.where)

            if not localboard.winned:
                localboard.checkwin()
                localboard.checkdraw()

            self.board[x][y] = localboard.winner

            self.checkwin()
            self.checkdraw()
            self.changeturn()

            return True

        return False

    def pop(self) -> None:
        ''' Revert board to the previous position. '''
        if not self.move_count:
            return

        self.where_history.pop()
        self.where = self.where_history[-1]
        
        localboard = self.localboards[self.where[0]][self.where[1]]
        localboard.pop()

        last_move = self.move_history[-1]

        self.draw = False
        self.ended = False
        self.winned = False
        self.winner = Piece()
        self.move_count -= 1
        self.move_history.pop()

        localboard.checkwin()
        localboard.checkdraw()

        self[last_move[0], last_move[1]] = localboard.winner
        
        self.revertturn()
        self.checkwin()
        self.checkdraw()

    def __str__(self) -> str:
        printed = []
        for (i, j) in self.moves_coords():
                printed.append(f'------------- LOCAL BOARD {i} {j} -------------\n')
                printed.append('Local attributes:')
                printed.append(str(self.localboards[i][j]))
        printed.append('\n=========== Global attributes: ===========')
        printed.append(super().__str__())
        return ''.join(printed)

==> pytactoe-0.10.2.tar.gz: pytactoe-0.10.2/pytactoe/variants.py <==
from pytactoe.board import Board, Piece
from pytactoe.tictactoe import TicTacToe


class WildTicTacToe(TicTacToe):
    ''' Tic Tac Toe where you can choose which piece to place. '''
    def __init__(self, m=3, n=3, k=3, infinite=False, first_player=Piece(1)) -> None:
        super().__init__(m, n, k, infinite, first_player)
        self.move_history_pieces = [] # Which pieces were chosen on each move

    def checkwin(self) -> bool:
        if super().checkwin():
            self.winner = self.move_count%(Piece.playernumber)+1

    def move(self, x, y, piece: Piece) -> bool:
        self.turn = piece
        if piece == Piece():
            return False
        self.move_history_pieces.append(piece)
        return super().move(x, y)


class MisereTicTacToe(TicTacToe):
    '''Misere TicTacToe, where the player wins if the opponent 'wins' in normal rules. '''
    def __init__(self, m=3, n=3, k=3, infinite=False, first_player: Piece = Piece(1)) -> None:
        super().__init__(m, n, k, infinite, first_player)
    
    def checkwin(self) -> bool:
        if super().checkwin():
            self.winner = Piece.previous(self.turn)


class ImpartialTicTacToe(TicTacToe):
    ''' Tic Tac Toe with only one piece type. '''
    def __init__(self, m=3, n=3, k=3, infinite=False) -> None:
        super().__init__(m, n, k, infinite)

    def move(self, x, y) -> bool:
        self.turn = Piece(1)
        super().move(x, y)


class UltimateTicTacToe(TicTacToe):
    ''' Ultimate TicTacToe. '''
    def __init__(self, m=3, n=3, k=3, infinite=False, first_player=Piece(1), where=(0, 0)) -> None:
        '''
        Initialize Ultimate TicTacToe.
        localboards attribute contain regular local boards; attribute .turn for local is unused.
        board attribute contain information about who won for each local board.
        where attribute determine on which local board the next move is pushed.
        where_history attribute contain all local boards coordinates.
        '''
        super().__init__(m, n, k, infinite, first_player)
        assert not infinite
        self.localboards = [[TicTacToe(m, n, k, infinite, first_player=Piece()) for _ in range(m)] for _ in range(n)]
        self.where = where
        self.where_history = [where]

    def movevalid(self, x, y) -> bool:
        ''' Return True if move is valid. Note that we can move on already winned small boards. '''
        return self.localboards[self.where[0]][self.where[1]].movevalid(x, y)

    def move(self, x, y) -> bool:
        '''Make move, update attributes, choose small board for next move. Return True if move is successful. '''
        if self.movevalid(x, y):

            localboard = self.localboards[self.where[0]][self.where[1]]
            localboard[x, y] = self.turn
            localboard.move_history.append((x,y))
            localboard.move_count += 1

            self.move_history.append((x,y))
            self.move_count += 1
            
            self.where = (x, y)
            self.where_history.append(self.where)

            if not localboard.winned:
                localboard.checkwin()
                localboard.checkdraw()

            self.board[x][y] = localboard.winner

            self.checkwin()
            self.checkdraw()
            self.changeturn()

            return True

        return False

    def pop(self) -> None:
        ''' Revert board to the previous position. '''
        if not self.move_count:
            return

        self.where_history.pop()
        self.where = self.where_history[-1]
        
        localboard = self.localboards[self.where[0]][self.where[1]]
        localboard.pop()

        last_move = self.move_history[-1]

        self.draw = False
        self.ended = False
        self.winned = False
        self.winner = Piece()
        self.move_count -= 1
        self.move_history.pop()

        localboard.checkwin()
        localboard.checkdraw()

        self[last_move[0], last_move[1]] = localboard.winner
        
        self.revertturn()
        self.checkwin()
        self.checkdraw()

    def __repr__(self) -> str:
        printed = []
        for (i, j) in self.moves_coords():
                printed.append(f'\n~~~~~ LOCAL BOARD {i} {j} ~~~~~\n')
                printed.append(str(self.localboards[i][j])+'\n')
        printed.append('\n===== GLOBAL BOARD =====\n')
        printed.append(super().__repr__()+f', where={self.where}')
        return ''.join(printed)
